# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "LuaService.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.luaservice


# structure
class ScriptState(Structure):
    idlType = "luaservice.ScriptState:1.0.0"
    elements = ["execState", "exitType", "exitStatus"]

    def __init__(self, execState, exitType, exitStatus):
        typecheck.is_enum(execState, raritan.rpc.luaservice.ScriptState.ExecState, AssertionError)
        typecheck.is_enum(exitType, raritan.rpc.luaservice.ScriptState.ExitType, AssertionError)
        typecheck.is_int(exitStatus, AssertionError)

        self.execState = execState
        self.exitType = exitType
        self.exitStatus = exitStatus

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            execState = raritan.rpc.luaservice.ScriptState.ExecState.decode(json['execState']),
            exitType = raritan.rpc.luaservice.ScriptState.ExitType.decode(json['exitType']),
            exitStatus = json['exitStatus'],
        )
        return obj

    def encode(self):
        json = {}
        json['execState'] = raritan.rpc.luaservice.ScriptState.ExecState.encode(self.execState)
        json['exitType'] = raritan.rpc.luaservice.ScriptState.ExitType.encode(self.exitType)
        json['exitStatus'] = self.exitStatus
        return json

    # enumeration
    class ExecState(Enumeration):
        idlType = "luaservice.ScriptState.ExecState:1.0.0"
        values = ["STAT_NEW", "STAT_RUNNING", "STAT_TERMINATED", "STAT_RESTARTING"]

    ExecState.STAT_NEW = ExecState(0)
    ExecState.STAT_RUNNING = ExecState(1)
    ExecState.STAT_TERMINATED = ExecState(2)
    ExecState.STAT_RESTARTING = ExecState(3)

    # enumeration
    class ExitType(Enumeration):
        idlType = "luaservice.ScriptState.ExitType:1.0.0"
        values = ["EXIT_CODE", "SIGNAL"]

    ExitType.EXIT_CODE = ExitType(0)
    ExitType.SIGNAL = ExitType(1)

# structure
class ScriptOptions(Structure):
    idlType = "luaservice.ScriptOptions:2.0.0"
    elements = ["defaultArgs", "autoStart", "autoRestart"]

    def __init__(self, defaultArgs, autoStart, autoRestart):
        typecheck.is_bool(autoStart, AssertionError)
        typecheck.is_bool(autoRestart, AssertionError)

        self.defaultArgs = defaultArgs
        self.autoStart = autoStart
        self.autoRestart = autoRestart

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            defaultArgs = dict([(
                elem['key'],
                elem['value'])
                for elem in json['defaultArgs']]),
            autoStart = json['autoStart'],
            autoRestart = json['autoRestart'],
        )
        return obj

    def encode(self):
        json = {}
        json['defaultArgs'] = [dict(
            key = k,
            value = v)
            for k, v in self.defaultArgs.items()]
        json['autoStart'] = self.autoStart
        json['autoRestart'] = self.autoRestart
        return json

# structure
class Environment(Structure):
    idlType = "luaservice.Environment:2.0.0"
    elements = ["maxScriptMemoryGrowth", "maxAmountOfScripts", "amountOfScripts", "maxScriptSize", "maxAllScriptSize", "allScriptSize", "outputBufferSize", "restartInterval", "autoStartDelay"]

    def __init__(self, maxScriptMemoryGrowth, maxAmountOfScripts, amountOfScripts, maxScriptSize, maxAllScriptSize, allScriptSize, outputBufferSize, restartInterval, autoStartDelay):
        typecheck.is_int(maxScriptMemoryGrowth, AssertionError)
        typecheck.is_int(maxAmountOfScripts, AssertionError)
        typecheck.is_int(amountOfScripts, AssertionError)
        typecheck.is_int(maxScriptSize, AssertionError)
        typecheck.is_int(maxAllScriptSize, AssertionError)
        typecheck.is_int(allScriptSize, AssertionError)
        typecheck.is_int(outputBufferSize, AssertionError)
        typecheck.is_int(restartInterval, AssertionError)
        typecheck.is_int(autoStartDelay, AssertionError)

        self.maxScriptMemoryGrowth = maxScriptMemoryGrowth
        self.maxAmountOfScripts = maxAmountOfScripts
        self.amountOfScripts = amountOfScripts
        self.maxScriptSize = maxScriptSize
        self.maxAllScriptSize = maxAllScriptSize
        self.allScriptSize = allScriptSize
        self.outputBufferSize = outputBufferSize
        self.restartInterval = restartInterval
        self.autoStartDelay = autoStartDelay

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            maxScriptMemoryGrowth = json['maxScriptMemoryGrowth'],
            maxAmountOfScripts = json['maxAmountOfScripts'],
            amountOfScripts = json['amountOfScripts'],
            maxScriptSize = json['maxScriptSize'],
            maxAllScriptSize = json['maxAllScriptSize'],
            allScriptSize = json['allScriptSize'],
            outputBufferSize = json['outputBufferSize'],
            restartInterval = json['restartInterval'],
            autoStartDelay = json['autoStartDelay'],
        )
        return obj

    def encode(self):
        json = {}
        json['maxScriptMemoryGrowth'] = self.maxScriptMemoryGrowth
        json['maxAmountOfScripts'] = self.maxAmountOfScripts
        json['amountOfScripts'] = self.amountOfScripts
        json['maxScriptSize'] = self.maxScriptSize
        json['maxAllScriptSize'] = self.maxAllScriptSize
        json['allScriptSize'] = self.allScriptSize
        json['outputBufferSize'] = self.outputBufferSize
        json['restartInterval'] = self.restartInterval
        json['autoStartDelay'] = self.autoStartDelay
        return json

# interface
class Manager(Interface):
    idlType = "luaservice.Manager:2.0.1"

    NO_ERROR = 0

    ERR_INVALID_NAME = 1

    ERR_NO_SUCH_SCRIPT = 2

    ERR_MAX_SCRIPT_NUMBERS_EXCEEDED = 3

    ERR_MAX_SCRIPT_SIZE_EXCEEDED = 4

    ERR_MAX_ALL_SCRIPT_SIZE_EXCEEDED = 5

    ERR_NOT_TERMINATED = 6

    ERR_NOT_RUNNING = 7

    ERR_INVALID_ADDR = 8

    ERR_TOO_MANY_ARGUMENTS = 10

    ERR_ARGUMENT_NOT_VALID = 11

    class _setScript(Interface.Method):
        name = 'setScript'

        @staticmethod
        def encode(name, script, options):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(script, AssertionError)
            typecheck.is_struct(options, raritan.rpc.luaservice.ScriptOptions, AssertionError)
            args = {}
            args['name'] = name
            args['script'] = script
            args['options'] = raritan.rpc.luaservice.ScriptOptions.encode(options)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getScript(Interface.Method):
        name = 'getScript'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            script = rsp['script']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_string(script, DecodeException)
            return (_ret_, script)

    class _getScriptNames(Interface.Method):
        name = 'getScriptNames'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_string(x0, DecodeException)
            return _ret_

    class _deleteScript(Interface.Method):
        name = 'deleteScript'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setScriptOptions(Interface.Method):
        name = 'setScriptOptions'

        @staticmethod
        def encode(name, options):
            typecheck.is_string(name, AssertionError)
            typecheck.is_struct(options, raritan.rpc.luaservice.ScriptOptions, AssertionError)
            args = {}
            args['name'] = name
            args['options'] = raritan.rpc.luaservice.ScriptOptions.encode(options)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getScriptOptions(Interface.Method):
        name = 'getScriptOptions'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            options = raritan.rpc.luaservice.ScriptOptions.decode(rsp['options'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(options, raritan.rpc.luaservice.ScriptOptions, DecodeException)
            return (_ret_, options)

    class _getEnvironment(Interface.Method):
        name = 'getEnvironment'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.luaservice.Environment.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.luaservice.Environment, DecodeException)
            return _ret_

    class _getScriptOutput(Interface.Method):
        name = 'getScriptOutput'

        @staticmethod
        def encode(name, iAddr):
            typecheck.is_string(name, AssertionError)
            typecheck.is_long(iAddr, AssertionError)
            args = {}
            args['name'] = name
            args['iAddr'] = iAddr
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            oAddr = int(rsp['oAddr'])
            nAddr = int(rsp['nAddr'])
            oString = rsp['oString']
            more = rsp['more']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_long(oAddr, DecodeException)
            typecheck.is_long(nAddr, DecodeException)
            typecheck.is_string(oString, DecodeException)
            typecheck.is_bool(more, DecodeException)
            return (_ret_, oAddr, nAddr, oString, more)

    class _clearScriptOutput(Interface.Method):
        name = 'clearScriptOutput'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _startScript(Interface.Method):
        name = 'startScript'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _startScriptWithArgs(Interface.Method):
        name = 'startScriptWithArgs'

        @staticmethod
        def encode(name, arguments):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            args['arguments'] = [dict(
                key = k,
                value = v)
                for k, v in arguments.items()]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _terminateScript(Interface.Method):
        name = 'terminateScript'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getScriptState(Interface.Method):
        name = 'getScriptState'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            state = raritan.rpc.luaservice.ScriptState.decode(rsp['state'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(state, raritan.rpc.luaservice.ScriptState, DecodeException)
            return (_ret_, state)

    class _getScriptStates(Interface.Method):
        name = 'getScriptStates'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = dict([(
                elem['key'],
                raritan.rpc.luaservice.ScriptState.decode(elem['value'], agent))
                for elem in rsp['_ret_']])
            return _ret_
    def __init__(self, target, agent):
        super(Manager, self).__init__(target, agent)
        self.setScript = Manager._setScript(self)
        self.getScript = Manager._getScript(self)
        self.getScriptNames = Manager._getScriptNames(self)
        self.deleteScript = Manager._deleteScript(self)
        self.setScriptOptions = Manager._setScriptOptions(self)
        self.getScriptOptions = Manager._getScriptOptions(self)
        self.getEnvironment = Manager._getEnvironment(self)
        self.getScriptOutput = Manager._getScriptOutput(self)
        self.clearScriptOutput = Manager._clearScriptOutput(self)
        self.startScript = Manager._startScript(self)
        self.startScriptWithArgs = Manager._startScriptWithArgs(self)
        self.terminateScript = Manager._terminateScript(self)
        self.getScriptState = Manager._getScriptState(self)
        self.getScriptStates = Manager._getScriptStates(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Role.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.usermgmt


# interface
class Role(Interface):
    idlType = "usermgmt.Role:1.0.0"

    ERR_INVALID_VALUE = 1

    # structure
    class Privilege(Structure):
        idlType = "usermgmt.Role.Privilege:1.0.0"
        elements = ["name", "args"]

        def __init__(self, name, args):
            typecheck.is_string(name, AssertionError)
            for x0 in args:
                typecheck.is_string(x0, AssertionError)

            self.name = name
            self.args = args

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                args = [x0 for x0 in json['args']],
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['args'] = [x0 for x0 in self.args]
            return json

    # structure
    class Info(Structure):
        idlType = "usermgmt.Role.Info:1.0.0"
        elements = ["description", "locked", "privileges"]

        def __init__(self, description, locked, privileges):
            typecheck.is_string(description, AssertionError)
            typecheck.is_bool(locked, AssertionError)
            for x0 in privileges:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.Role.Privilege, AssertionError)

            self.description = description
            self.locked = locked
            self.privileges = privileges

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                description = json['description'],
                locked = json['locked'],
                privileges = [raritan.rpc.usermgmt.Role.Privilege.decode(x0, agent) for x0 in json['privileges']],
            )
            return obj

        def encode(self):
            json = {}
            json['description'] = self.description
            json['locked'] = self.locked
            json['privileges'] = [raritan.rpc.usermgmt.Role.Privilege.encode(x0) for x0 in self.privileges]
            return json

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.usermgmt.Role.Info.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.usermgmt.Role.Info, DecodeException)
            return _ret_

    class _updateFull(Interface.Method):
        name = 'updateFull'

        @staticmethod
        def encode(info):
            typecheck.is_struct(info, raritan.rpc.usermgmt.Role.Info, AssertionError)
            args = {}
            args['info'] = raritan.rpc.usermgmt.Role.Info.encode(info)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Role, self).__init__(target, agent)
        self.getInfo = Role._getInfo(self)
        self.updateFull = Role._updateFull(self)

#
# Section generated by IdlC from "RoleManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.usermgmt


# value object
class RoleEvent(raritan.rpc.event.UserEvent):
    idlType = "usermgmt.RoleEvent:1.0.0"

    def __init__(self, rolename, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.RoleEvent, self).__init__(actUserName, actIpAddr, source)
        typecheck.is_string(rolename, AssertionError)

        self.rolename = rolename

    def encode(self):
        json = super(raritan.rpc.usermgmt.RoleEvent, self).encode()
        json['rolename'] = self.rolename
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            rolename = json['rolename'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["rolename"]
        elements = elements + super(raritan.rpc.usermgmt.RoleEvent, self).listElements()
        return elements

# value object
class RoleAdded(RoleEvent):
    idlType = "usermgmt.RoleAdded:1.0.0"

    def __init__(self, rolename, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.RoleAdded, self).__init__(rolename, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.usermgmt.RoleAdded, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for usermgmt.RoleEvent
            rolename = json['rolename'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.usermgmt.RoleAdded, self).listElements()
        return elements

# value object
class RoleRemoved(RoleEvent):
    idlType = "usermgmt.RoleRemoved:1.0.0"

    def __init__(self, rolename, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.RoleRemoved, self).__init__(rolename, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.usermgmt.RoleRemoved, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for usermgmt.RoleEvent
            rolename = json['rolename'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.usermgmt.RoleRemoved, self).listElements()
        return elements

# value object
class RoleChanged(RoleEvent):
    idlType = "usermgmt.RoleChanged:1.0.0"

    def __init__(self, oldSettings, newSettings, rolename, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.RoleChanged, self).__init__(rolename, actUserName, actIpAddr, source)
        typecheck.is_struct(oldSettings, raritan.rpc.usermgmt.Role.Info, AssertionError)
        typecheck.is_struct(newSettings, raritan.rpc.usermgmt.Role.Info, AssertionError)

        self.oldSettings = oldSettings
        self.newSettings = newSettings

    def encode(self):
        json = super(raritan.rpc.usermgmt.RoleChanged, self).encode()
        json['oldSettings'] = raritan.rpc.usermgmt.Role.Info.encode(self.oldSettings)
        json['newSettings'] = raritan.rpc.usermgmt.Role.Info.encode(self.newSettings)
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            oldSettings = raritan.rpc.usermgmt.Role.Info.decode(json['oldSettings'], agent),
            newSettings = raritan.rpc.usermgmt.Role.Info.decode(json['newSettings'], agent),
            # for usermgmt.RoleEvent
            rolename = json['rolename'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["oldSettings", "newSettings"]
        elements = elements + super(raritan.rpc.usermgmt.RoleChanged, self).listElements()
        return elements

# interface
class RoleManager(Interface):
    idlType = "usermgmt.RoleManager:1.0.0"

    ERR_ROLE_ALREADY_EXISTS = 1

    ERR_MAX_ROLES_REACHED = 2

    ERR_INVALID_VALUE = 3

    ERR_ROLE_DOESNT_EXIST = 1

    ERR_ROLE_NOT_DELETABLE = 2

    # structure
    class ArgumentDesc(Structure):
        idlType = "usermgmt.RoleManager.ArgumentDesc:1.0.0"
        elements = ["name", "desc"]

        def __init__(self, name, desc):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(desc, AssertionError)

            self.name = name
            self.desc = desc

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                desc = json['desc'],
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['desc'] = self.desc
            return json

    # structure
    class PrivilegeDesc(Structure):
        idlType = "usermgmt.RoleManager.PrivilegeDesc:1.0.0"
        elements = ["name", "desc", "args"]

        def __init__(self, name, desc, args):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(desc, AssertionError)
            for x0 in args:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.RoleManager.ArgumentDesc, AssertionError)

            self.name = name
            self.desc = desc
            self.args = args

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                desc = json['desc'],
                args = [raritan.rpc.usermgmt.RoleManager.ArgumentDesc.decode(x0, agent) for x0 in json['args']],
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['desc'] = self.desc
            json['args'] = [raritan.rpc.usermgmt.RoleManager.ArgumentDesc.encode(x0) for x0 in self.args]
            return json

    # structure
    class RoleAccount(Structure):
        idlType = "usermgmt.RoleManager.RoleAccount:1.0.0"
        elements = ["id", "name", "info"]

        def __init__(self, id, name, info):
            typecheck.is_int(id, AssertionError)
            typecheck.is_string(name, AssertionError)
            typecheck.is_struct(info, raritan.rpc.usermgmt.Role.Info, AssertionError)

            self.id = id
            self.name = name
            self.info = info

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                name = json['name'],
                info = raritan.rpc.usermgmt.Role.Info.decode(json['info'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = self.id
            json['name'] = self.name
            json['info'] = raritan.rpc.usermgmt.Role.Info.encode(self.info)
            return json

    # structure
    class Info(Structure):
        idlType = "usermgmt.RoleManager.Info:1.0.0"
        elements = ["privileges", "roles"]

        def __init__(self, privileges, roles):
            for x0 in privileges:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.RoleManager.PrivilegeDesc, AssertionError)
            for x0 in roles:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.RoleManager.RoleAccount, AssertionError)

            self.privileges = privileges
            self.roles = roles

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                privileges = [raritan.rpc.usermgmt.RoleManager.PrivilegeDesc.decode(x0, agent) for x0 in json['privileges']],
                roles = [raritan.rpc.usermgmt.RoleManager.RoleAccount.decode(x0, agent) for x0 in json['roles']],
            )
            return obj

        def encode(self):
            json = {}
            json['privileges'] = [raritan.rpc.usermgmt.RoleManager.PrivilegeDesc.encode(x0) for x0 in self.privileges]
            json['roles'] = [raritan.rpc.usermgmt.RoleManager.RoleAccount.encode(x0) for x0 in self.roles]
            return json

    class _createRoleFull(Interface.Method):
        name = 'createRoleFull'

        @staticmethod
        def encode(name, info):
            typecheck.is_string(name, AssertionError)
            typecheck.is_struct(info, raritan.rpc.usermgmt.Role.Info, AssertionError)
            args = {}
            args['name'] = name
            args['info'] = raritan.rpc.usermgmt.Role.Info.encode(info)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _deleteRole(Interface.Method):
        name = 'deleteRole'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getAllRoleNames(Interface.Method):
        name = 'getAllRoleNames'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_string(x0, DecodeException)
            return _ret_

    class _getAllRoles(Interface.Method):
        name = 'getAllRoles'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.usermgmt.RoleManager.RoleAccount.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.RoleManager.RoleAccount, DecodeException)
            return _ret_

    class _getAllPrivileges(Interface.Method):
        name = 'getAllPrivileges'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.usermgmt.RoleManager.PrivilegeDesc.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.RoleManager.PrivilegeDesc, DecodeException)
            return _ret_

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.usermgmt.RoleManager.Info.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.usermgmt.RoleManager.Info, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(RoleManager, self).__init__(target, agent)
        self.createRoleFull = RoleManager._createRoleFull(self)
        self.deleteRole = RoleManager._deleteRole(self)
        self.getAllRoleNames = RoleManager._getAllRoleNames(self)
        self.getAllRoles = RoleManager._getAllRoles(self)
        self.getAllPrivileges = RoleManager._getAllPrivileges(self)
        self.getInfo = RoleManager._getInfo(self)

#
# Section generated by IdlC from "User.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.um

import raritan.rpc.usermgmt


# structure
class SnmpV3Settings(Structure):
    idlType = "usermgmt.SnmpV3Settings:1.0.0"
    elements = ["enabled", "secLevel", "authProtocol", "usePasswordAsAuthPassphrase", "haveAuthPassphrase", "authPassphrase", "privProtocol", "useAuthPassphraseAsPrivPassphrase", "havePrivPassphrase", "privPassphrase"]

    def __init__(self, enabled, secLevel, authProtocol, usePasswordAsAuthPassphrase, haveAuthPassphrase, authPassphrase, privProtocol, useAuthPassphraseAsPrivPassphrase, havePrivPassphrase, privPassphrase):
        typecheck.is_bool(enabled, AssertionError)
        typecheck.is_enum(secLevel, raritan.rpc.um.SnmpV3.SecurityLevel, AssertionError)
        typecheck.is_enum(authProtocol, raritan.rpc.um.SnmpV3.AuthProtocol, AssertionError)
        typecheck.is_bool(usePasswordAsAuthPassphrase, AssertionError)
        typecheck.is_bool(haveAuthPassphrase, AssertionError)
        typecheck.is_string(authPassphrase, AssertionError)
        typecheck.is_enum(privProtocol, raritan.rpc.um.SnmpV3.PrivProtocol, AssertionError)
        typecheck.is_bool(useAuthPassphraseAsPrivPassphrase, AssertionError)
        typecheck.is_bool(havePrivPassphrase, AssertionError)
        typecheck.is_string(privPassphrase, AssertionError)

        self.enabled = enabled
        self.secLevel = secLevel
        self.authProtocol = authProtocol
        self.usePasswordAsAuthPassphrase = usePasswordAsAuthPassphrase
        self.haveAuthPassphrase = haveAuthPassphrase
        self.authPassphrase = authPassphrase
        self.privProtocol = privProtocol
        self.useAuthPassphraseAsPrivPassphrase = useAuthPassphraseAsPrivPassphrase
        self.havePrivPassphrase = havePrivPassphrase
        self.privPassphrase = privPassphrase

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            enabled = json['enabled'],
            secLevel = raritan.rpc.um.SnmpV3.SecurityLevel.decode(json['secLevel']),
            authProtocol = raritan.rpc.um.SnmpV3.AuthProtocol.decode(json['authProtocol']),
            usePasswordAsAuthPassphrase = json['usePasswordAsAuthPassphrase'],
            haveAuthPassphrase = json['haveAuthPassphrase'],
            authPassphrase = json['authPassphrase'],
            privProtocol = raritan.rpc.um.SnmpV3.PrivProtocol.decode(json['privProtocol']),
            useAuthPassphraseAsPrivPassphrase = json['useAuthPassphraseAsPrivPassphrase'],
            havePrivPassphrase = json['havePrivPassphrase'],
            privPassphrase = json['privPassphrase'],
        )
        return obj

    def encode(self):
        json = {}
        json['enabled'] = self.enabled
        json['secLevel'] = raritan.rpc.um.SnmpV3.SecurityLevel.encode(self.secLevel)
        json['authProtocol'] = raritan.rpc.um.SnmpV3.AuthProtocol.encode(self.authProtocol)
        json['usePasswordAsAuthPassphrase'] = self.usePasswordAsAuthPassphrase
        json['haveAuthPassphrase'] = self.haveAuthPassphrase
        json['authPassphrase'] = self.authPassphrase
        json['privProtocol'] = raritan.rpc.um.SnmpV3.PrivProtocol.encode(self.privProtocol)
        json['useAuthPassphraseAsPrivPassphrase'] = self.useAuthPassphraseAsPrivPassphrase
        json['havePrivPassphrase'] = self.havePrivPassphrase
        json['privPassphrase'] = self.privPassphrase
        return json

# structure
class AuxInfo(Structure):
    idlType = "usermgmt.AuxInfo:1.0.0"
    elements = ["fullname", "telephone", "eMail"]

    def __init__(self, fullname, telephone, eMail):
        typecheck.is_string(fullname, AssertionError)
        typecheck.is_string(telephone, AssertionError)
        typecheck.is_string(eMail, AssertionError)

        self.fullname = fullname
        self.telephone = telephone
        self.eMail = eMail

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            fullname = json['fullname'],
            telephone = json['telephone'],
            eMail = json['eMail'],
        )
        return obj

    def encode(self):
        json = {}
        json['fullname'] = self.fullname
        json['telephone'] = self.telephone
        json['eMail'] = self.eMail
        return json

# enumeration
class TemperatureEnum(Enumeration):
    idlType = "usermgmt.TemperatureEnum:1.0.0"
    values = ["DEG_C", "DEG_F"]

TemperatureEnum.DEG_C = TemperatureEnum(0)
TemperatureEnum.DEG_F = TemperatureEnum(1)

# enumeration
class LengthEnum(Enumeration):
    idlType = "usermgmt.LengthEnum:1.0.0"
    values = ["METER", "FEET"]

LengthEnum.METER = LengthEnum(0)
LengthEnum.FEET = LengthEnum(1)

# enumeration
class PressureEnum(Enumeration):
    idlType = "usermgmt.PressureEnum:1.0.0"
    values = ["PASCAL", "PSI"]

PressureEnum.PASCAL = PressureEnum(0)
PressureEnum.PSI = PressureEnum(1)

# structure
class Preferences(Structure):
    idlType = "usermgmt.Preferences:1.0.0"
    elements = ["temperatureUnit", "lengthUnit", "pressureUnit"]

    def __init__(self, temperatureUnit, lengthUnit, pressureUnit):
        typecheck.is_enum(temperatureUnit, raritan.rpc.usermgmt.TemperatureEnum, AssertionError)
        typecheck.is_enum(lengthUnit, raritan.rpc.usermgmt.LengthEnum, AssertionError)
        typecheck.is_enum(pressureUnit, raritan.rpc.usermgmt.PressureEnum, AssertionError)

        self.temperatureUnit = temperatureUnit
        self.lengthUnit = lengthUnit
        self.pressureUnit = pressureUnit

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            temperatureUnit = raritan.rpc.usermgmt.TemperatureEnum.decode(json['temperatureUnit']),
            lengthUnit = raritan.rpc.usermgmt.LengthEnum.decode(json['lengthUnit']),
            pressureUnit = raritan.rpc.usermgmt.PressureEnum.decode(json['pressureUnit']),
        )
        return obj

    def encode(self):
        json = {}
        json['temperatureUnit'] = raritan.rpc.usermgmt.TemperatureEnum.encode(self.temperatureUnit)
        json['lengthUnit'] = raritan.rpc.usermgmt.LengthEnum.encode(self.lengthUnit)
        json['pressureUnit'] = raritan.rpc.usermgmt.PressureEnum.encode(self.pressureUnit)
        return json

# structure
class UserInfo(Structure):
    idlType = "usermgmt.UserInfo:1.0.0"
    elements = ["enabled", "locked", "blocked", "needPasswordChange", "auxInfo", "snmpV3Settings", "sshPublicKey", "preferences", "roleIds"]

    def __init__(self, enabled, locked, blocked, needPasswordChange, auxInfo, snmpV3Settings, sshPublicKey, preferences, roleIds):
        typecheck.is_bool(enabled, AssertionError)
        typecheck.is_bool(locked, AssertionError)
        typecheck.is_bool(blocked, AssertionError)
        typecheck.is_bool(needPasswordChange, AssertionError)
        typecheck.is_struct(auxInfo, raritan.rpc.usermgmt.AuxInfo, AssertionError)
        typecheck.is_struct(snmpV3Settings, raritan.rpc.usermgmt.SnmpV3Settings, AssertionError)
        typecheck.is_string(sshPublicKey, AssertionError)
        typecheck.is_struct(preferences, raritan.rpc.usermgmt.Preferences, AssertionError)
        for x0 in roleIds:
            typecheck.is_int(x0, AssertionError)

        self.enabled = enabled
        self.locked = locked
        self.blocked = blocked
        self.needPasswordChange = needPasswordChange
        self.auxInfo = auxInfo
        self.snmpV3Settings = snmpV3Settings
        self.sshPublicKey = sshPublicKey
        self.preferences = preferences
        self.roleIds = roleIds

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            enabled = json['enabled'],
            locked = json['locked'],
            blocked = json['blocked'],
            needPasswordChange = json['needPasswordChange'],
            auxInfo = raritan.rpc.usermgmt.AuxInfo.decode(json['auxInfo'], agent),
            snmpV3Settings = raritan.rpc.usermgmt.SnmpV3Settings.decode(json['snmpV3Settings'], agent),
            sshPublicKey = json['sshPublicKey'],
            preferences = raritan.rpc.usermgmt.Preferences.decode(json['preferences'], agent),
            roleIds = [x0 for x0 in json['roleIds']],
        )
        return obj

    def encode(self):
        json = {}
        json['enabled'] = self.enabled
        json['locked'] = self.locked
        json['blocked'] = self.blocked
        json['needPasswordChange'] = self.needPasswordChange
        json['auxInfo'] = raritan.rpc.usermgmt.AuxInfo.encode(self.auxInfo)
        json['snmpV3Settings'] = raritan.rpc.usermgmt.SnmpV3Settings.encode(self.snmpV3Settings)
        json['sshPublicKey'] = self.sshPublicKey
        json['preferences'] = raritan.rpc.usermgmt.Preferences.encode(self.preferences)
        json['roleIds'] = [x0 for x0 in self.roleIds]
        return json

# structure
class UserCapabilities(Structure):
    idlType = "usermgmt.UserCapabilities:1.0.0"
    elements = ["canSetPassword", "canSetPreferences"]

    def __init__(self, canSetPassword, canSetPreferences):
        typecheck.is_bool(canSetPassword, AssertionError)
        typecheck.is_bool(canSetPreferences, AssertionError)

        self.canSetPassword = canSetPassword
        self.canSetPreferences = canSetPreferences

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            canSetPassword = json['canSetPassword'],
            canSetPreferences = json['canSetPreferences'],
        )
        return obj

    def encode(self):
        json = {}
        json['canSetPassword'] = self.canSetPassword
        json['canSetPreferences'] = self.canSetPreferences
        return json

# interface
class User(Interface):
    idlType = "usermgmt.User:1.0.2"

    ERR_PASSWORD_UNCHANGED = 1

    ERR_PASSWORD_EMPTY = 2

    ERR_PASSWORD_TOO_SHORT = 3

    ERR_PASSWORD_TOO_LONG = 4

    ERR_PASSWORD_CTRL_CHARS = 5

    ERR_PASSWORD_NEED_LOWER = 6

    ERR_PASSWORD_NEED_UPPER = 7

    ERR_PASSWORD_NEED_NUMERIC = 8

    ERR_PASSWORD_NEED_SPECIAL = 9

    ERR_PASSWORD_IN_HISTORY = 10

    ERR_PASSWORD_TOO_SHORT_FOR_SNMP = 11

    ERR_INVALID_ARGUMENT = 12

    ERR_WRONG_PASSWORD = 13

    ERR_SSH_PUBKEY_DATA_TOO_LARGE = 14

    ERR_SSH_PUBKEY_INVALID = 15

    ERR_SSH_PUBKEY_NOT_SUPPORTED = 16

    ERR_SSH_RSA_PUBKEY_TOO_SHORT = 17

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.usermgmt.UserInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.usermgmt.UserInfo, DecodeException)
            return _ret_

    class _setAccountPassword(Interface.Method):
        name = 'setAccountPassword'

        @staticmethod
        def encode(password):
            typecheck.is_string(password, AssertionError)
            args = {}
            args['password'] = password
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _updateAccountFull(Interface.Method):
        name = 'updateAccountFull'

        @staticmethod
        def encode(password, info):
            typecheck.is_string(password, AssertionError)
            typecheck.is_struct(info, raritan.rpc.usermgmt.UserInfo, AssertionError)
            args = {}
            args['password'] = password
            args['info'] = raritan.rpc.usermgmt.UserInfo.encode(info)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getInfoAndPrivileges(Interface.Method):
        name = 'getInfoAndPrivileges'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            info = raritan.rpc.usermgmt.UserInfo.decode(rsp['info'], agent)
            privileges = [raritan.rpc.usermgmt.Role.Privilege.decode(x0, agent) for x0 in rsp['privileges']]
            typecheck.is_struct(info, raritan.rpc.usermgmt.UserInfo, DecodeException)
            for x0 in privileges:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.Role.Privilege, DecodeException)
            return (info, privileges)

    class _setPreferences(Interface.Method):
        name = 'setPreferences'

        @staticmethod
        def encode(prefs):
            typecheck.is_struct(prefs, raritan.rpc.usermgmt.Preferences, AssertionError)
            args = {}
            args['prefs'] = raritan.rpc.usermgmt.Preferences.encode(prefs)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getCapabilities(Interface.Method):
        name = 'getCapabilities'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.usermgmt.UserCapabilities.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.usermgmt.UserCapabilities, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(User, self).__init__(target, agent)
        self.getInfo = User._getInfo(self)
        self.setAccountPassword = User._setAccountPassword(self)
        self.updateAccountFull = User._updateAccountFull(self)
        self.getInfoAndPrivileges = User._getInfoAndPrivileges(self)
        self.setPreferences = User._setPreferences(self)
        self.getCapabilities = User._getCapabilities(self)

#
# Section generated by IdlC from "UserManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.usermgmt


# structure
class Account(Structure):
    idlType = "usermgmt.Account:1.0.0"
    elements = ["name", "info"]

    def __init__(self, name, info):
        typecheck.is_string(name, AssertionError)
        typecheck.is_struct(info, raritan.rpc.usermgmt.UserInfo, AssertionError)

        self.name = name
        self.info = info

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            name = json['name'],
            info = raritan.rpc.usermgmt.UserInfo.decode(json['info'], agent),
        )
        return obj

    def encode(self):
        json = {}
        json['name'] = self.name
        json['info'] = raritan.rpc.usermgmt.UserInfo.encode(self.info)
        return json

# value object
class AccountEvent(raritan.rpc.event.UserEvent):
    idlType = "usermgmt.AccountEvent:1.0.0"

    def __init__(self, username, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.AccountEvent, self).__init__(actUserName, actIpAddr, source)
        typecheck.is_string(username, AssertionError)

        self.username = username

    def encode(self):
        json = super(raritan.rpc.usermgmt.AccountEvent, self).encode()
        json['username'] = self.username
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            username = json['username'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["username"]
        elements = elements + super(raritan.rpc.usermgmt.AccountEvent, self).listElements()
        return elements

# value object
class AccountAdded(AccountEvent):
    idlType = "usermgmt.AccountAdded:1.0.0"

    def __init__(self, username, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.AccountAdded, self).__init__(username, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.usermgmt.AccountAdded, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for usermgmt.AccountEvent
            username = json['username'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.usermgmt.AccountAdded, self).listElements()
        return elements

# value object
class AccountRemoved(AccountEvent):
    idlType = "usermgmt.AccountRemoved:1.0.0"

    def __init__(self, username, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.AccountRemoved, self).__init__(username, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.usermgmt.AccountRemoved, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for usermgmt.AccountEvent
            username = json['username'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.usermgmt.AccountRemoved, self).listElements()
        return elements

# value object
class PasswordChanged(AccountEvent):
    idlType = "usermgmt.PasswordChanged:1.0.0"

    def __init__(self, username, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.PasswordChanged, self).__init__(username, actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.usermgmt.PasswordChanged, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for usermgmt.AccountEvent
            username = json['username'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.usermgmt.PasswordChanged, self).listElements()
        return elements

# value object
class AccountChanged(AccountEvent):
    idlType = "usermgmt.AccountChanged:1.0.0"

    def __init__(self, oldSettings, newSettings, username, actUserName, actIpAddr, source):
        super(raritan.rpc.usermgmt.AccountChanged, self).__init__(username, actUserName, actIpAddr, source)
        typecheck.is_struct(oldSettings, raritan.rpc.usermgmt.UserInfo, AssertionError)
        typecheck.is_struct(newSettings, raritan.rpc.usermgmt.UserInfo, AssertionError)

        self.oldSettings = oldSettings
        self.newSettings = newSettings

    def encode(self):
        json = super(raritan.rpc.usermgmt.AccountChanged, self).encode()
        json['oldSettings'] = raritan.rpc.usermgmt.UserInfo.encode(self.oldSettings)
        json['newSettings'] = raritan.rpc.usermgmt.UserInfo.encode(self.newSettings)
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            oldSettings = raritan.rpc.usermgmt.UserInfo.decode(json['oldSettings'], agent),
            newSettings = raritan.rpc.usermgmt.UserInfo.decode(json['newSettings'], agent),
            # for usermgmt.AccountEvent
            username = json['username'],
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["oldSettings", "newSettings"]
        elements = elements + super(raritan.rpc.usermgmt.AccountChanged, self).listElements()
        return elements

# interface
class UserManager(Interface):
    idlType = "usermgmt.UserManager:1.0.4"

    ERR_USER_DOESNT_EXIST = 1

    ERR_USER_NOT_DELETABLE = 2

    ERR_USER_ALREADY_EXISTS = 1

    ERR_MAX_USERS_REACHED = 2

    ERR_PASSWORD_TOO_SHORT_FOR_SNMP = 3

    ERR_INVALID_VALUE = 4

    ERR_PASSWORD_EMPTY = 5

    ERR_PASSWORD_TOO_SHORT = 6

    ERR_PASSWORD_TOO_LONG = 7

    ERR_PASSWORD_CTRL_CHARS = 8

    ERR_PASSWORD_NEED_LOWER = 9

    ERR_PASSWORD_NEED_UPPER = 10

    ERR_PASSWORD_NEED_NUMERIC = 11

    ERR_PASSWORD_NEED_SPECIAL = 12

    ERR_SSH_PUBKEY_DATA_TOO_LARGE = 14

    ERR_SSH_PUBKEY_INVALID = 15

    ERR_SSH_PUBKEY_NOT_SUPPORTED = 16

    ERR_SSH_RSA_PUBKEY_TOO_SHORT = 17

    ERR_USERNAME_INVALID = 18

    class _getAccountNames(Interface.Method):
        name = 'getAccountNames'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_string(x0, DecodeException)
            return _ret_

    class _createAccount(Interface.Method):
        name = 'createAccount'

        @staticmethod
        def encode(username, password):
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)
            args = {}
            args['username'] = username
            args['password'] = password
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _deleteAccount(Interface.Method):
        name = 'deleteAccount'

        @staticmethod
        def encode(username):
            typecheck.is_string(username, AssertionError)
            args = {}
            args['username'] = username
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getAllAccounts(Interface.Method):
        name = 'getAllAccounts'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.usermgmt.Account.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.Account, DecodeException)
            return _ret_

    class _createAccountFull(Interface.Method):
        name = 'createAccountFull'

        @staticmethod
        def encode(username, password, info):
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)
            typecheck.is_struct(info, raritan.rpc.usermgmt.UserInfo, AssertionError)
            args = {}
            args['username'] = username
            args['password'] = password
            args['info'] = raritan.rpc.usermgmt.UserInfo.encode(info)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getAccountsByRole(Interface.Method):
        name = 'getAccountsByRole'

        @staticmethod
        def encode(roleName):
            typecheck.is_string(roleName, AssertionError)
            args = {}
            args['roleName'] = roleName
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.usermgmt.Account.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.usermgmt.Account, DecodeException)
            return _ret_

    class _getDefaultPreferences(Interface.Method):
        name = 'getDefaultPreferences'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.usermgmt.Preferences.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.usermgmt.Preferences, DecodeException)
            return _ret_

    class _setDefaultPreferences(Interface.Method):
        name = 'setDefaultPreferences'

        @staticmethod
        def encode(prefs):
            typecheck.is_struct(prefs, raritan.rpc.usermgmt.Preferences, AssertionError)
            args = {}
            args['prefs'] = raritan.rpc.usermgmt.Preferences.encode(prefs)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(UserManager, self).__init__(target, agent)
        self.getAccountNames = UserManager._getAccountNames(self)
        self.createAccount = UserManager._createAccount(self)
        self.deleteAccount = UserManager._deleteAccount(self)
        self.getAllAccounts = UserManager._getAllAccounts(self)
        self.createAccountFull = UserManager._createAccountFull(self)
        self.getAccountsByRole = UserManager._getAccountsByRole(self)
        self.getDefaultPreferences = UserManager._getDefaultPreferences(self)
        self.setDefaultPreferences = UserManager._setDefaultPreferences(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "DisplayControl.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.display

import raritan.rpc.event


# interface
class DisplayControl(Interface):
    idlType = "display.DisplayControl:1.0.1"

    SUCCESS = 0

    ERR_INVALID_VALUE = 1

    # structure
    class Info(Structure):
        idlType = "display.DisplayControl_1_0_1.Info:1.0.0"
        elements = ["fwAppVersion", "fwBootVersion", "orientation", "width", "height", "versionAvailable"]

        def __init__(self, fwAppVersion, fwBootVersion, orientation, width, height, versionAvailable):
            typecheck.is_int(fwAppVersion, AssertionError)
            typecheck.is_int(fwBootVersion, AssertionError)
            typecheck.is_int(orientation, AssertionError)
            typecheck.is_int(width, AssertionError)
            typecheck.is_int(height, AssertionError)
            typecheck.is_bool(versionAvailable, AssertionError)

            self.fwAppVersion = fwAppVersion
            self.fwBootVersion = fwBootVersion
            self.orientation = orientation
            self.width = width
            self.height = height
            self.versionAvailable = versionAvailable

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                fwAppVersion = json['fwAppVersion'],
                fwBootVersion = json['fwBootVersion'],
                orientation = json['orientation'],
                width = json['width'],
                height = json['height'],
                versionAvailable = json['versionAvailable'],
            )
            return obj

        def encode(self):
            json = {}
            json['fwAppVersion'] = self.fwAppVersion
            json['fwBootVersion'] = self.fwBootVersion
            json['orientation'] = self.orientation
            json['width'] = self.width
            json['height'] = self.height
            json['versionAvailable'] = self.versionAvailable
            return json

    # structure
    class DefaultViewItem(Structure):
        idlType = "display.DisplayControl_1_0_1.DefaultViewItem:1.0.0"
        elements = ["id", "description"]

        def __init__(self, id, description):
            typecheck.is_string(id, AssertionError)
            typecheck.is_string(description, AssertionError)

            self.id = id
            self.description = description

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                description = json['description'],
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = self.id
            json['description'] = self.description
            return json

    # value object
    class DefaultViewChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "display.DisplayControl_1_0_1.DefaultViewChangedEvent:1.0.0"

        def __init__(self, newView, actUserName, actIpAddr, source):
            super(raritan.rpc.display.DisplayControl.DefaultViewChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(newView, raritan.rpc.display.DisplayControl.DefaultViewItem, AssertionError)

            self.newView = newView

        def encode(self):
            json = super(raritan.rpc.display.DisplayControl.DefaultViewChangedEvent, self).encode()
            json['newView'] = raritan.rpc.display.DisplayControl.DefaultViewItem.encode(self.newView)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                newView = raritan.rpc.display.DisplayControl.DefaultViewItem.decode(json['newView'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["newView"]
            elements = elements + super(raritan.rpc.display.DisplayControl.DefaultViewChangedEvent, self).listElements()
            return elements

    class _getAvailableDefaultViews(Interface.Method):
        name = 'getAvailableDefaultViews'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.display.DisplayControl.DefaultViewItem.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.display.DisplayControl.DefaultViewItem, DecodeException)
            return _ret_

    class _getDefaultView(Interface.Method):
        name = 'getDefaultView'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _setDefaultView(Interface.Method):
        name = 'setDefaultView'

        @staticmethod
        def encode(id):
            typecheck.is_string(id, AssertionError)
            args = {}
            args['id'] = id
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.display.DisplayControl.Info.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.display.DisplayControl.Info, DecodeException)
            return _ret_

    class _showSmiley(Interface.Method):
        name = 'showSmiley'

        @staticmethod
        def encode(on, happy, msg):
            typecheck.is_bool(on, AssertionError)
            typecheck.is_bool(happy, AssertionError)
            typecheck.is_string(msg, AssertionError)
            args = {}
            args['on'] = on
            args['happy'] = happy
            args['msg'] = msg
            return args

        @staticmethod
        def decode(rsp, agent):
            return None
    def __init__(self, target, agent):
        super(DisplayControl, self).__init__(target, agent)
        self.getAvailableDefaultViews = DisplayControl._getAvailableDefaultViews(self)
        self.getDefaultView = DisplayControl._getDefaultView(self)
        self.setDefaultView = DisplayControl._setDefaultView(self)
        self.getInfo = DisplayControl._getInfo(self)
        self.showSmiley = DisplayControl._showSmiley(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "NameService.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# interface
class NameService(Interface):
    idlType = "jsonrpc.NameService:1.0.0"

    class _lookup(Interface.Method):
        name = 'lookup'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_remote_obj(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(NameService, self).__init__(target, agent)
        self.lookup = NameService._lookup(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2010 Raritan Inc. All rights reserved.

#
# relative imports
#
from .Interface import Interface
from .Enumeration import Enumeration
from .Structure import Structure
from .Agent import Agent
from .TypeInfo import TypeInfo
from .Time import Time
from .Utils import Utils
from .ValueObject import ValueObject

#
# exceptions
#

class HttpException(Exception):
    pass

class JsonRpcSyntaxException(Exception):
    pass

class JsonRpcErrorException(Exception):
    pass

class DecodeException(Exception):
    pass

from .BulkRequestHelper import BulkRequestHelper



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2010 Raritan Inc. All rights reserved.

from __future__ import absolute_import
import sys

import raritan.rpc
from raritan.rpc import Interface, Structure, Enumeration, ValueObject
import inspect

#
# simple types
#
def is_bool(x, exception):
    if not isinstance(x, bool):
        raise exception("'%s' is not a bool" % x)

def _is_int(x):
    if sys.version_info.major < 3:
        return isinstance(x, (int, long))
    else:
        return isinstance(x, int)

def _is_float(x):
    return _is_int(x) or isinstance(x, float)

def is_byte(x, exception):
    if not _is_int(x) or int(x) < 0 or int(x) > 255:
        raise exception("'%s' is not a byte" % x)

def is_int(x, exception):
    if not _is_int(x):
        raise exception("'%s' is not an integer" % x)

def is_long(x, exception):
    if not _is_int(x):
        raise exception("'%s' is not a long" % x)

def is_float(x, exception):
    if not _is_float(x):
        raise exception("'%s' is not a float" % x)

def is_double(x, exception):
    if not _is_float(x):
        raise exception("'%s' is not a double" % x)

def is_string(x, exception):
    if sys.version_info.major < 3:
        if not isinstance(x, basestring):
            raise exception("'%s' is not a string" % x)
    else:
        if not isinstance(x, str):
            raise exception("'%s' is not a string" % x)

def is_time(x, exception):
    if not isinstance(x, raritan.rpc.Time):
        raise exception("'%s' is not a Time" % x)

def is_remote_obj(x, exception):
    is_interface(x, Interface, exception)

def is_typeinfo(x, exception):
    if not inspect.isclass(x) or (not issubclass(x, Interface) and not issubclass(x, ValueObject)):
        raise exception("'%s' is not a TypeInfo" % x)

#
# complex types
#
def is_interface(x, cls, exception):
    # allow to pass None as interface
    if x is None:
        return True

    if not isinstance(x, Interface):
        raise exception("'%s' is not an interface reference" % (x))
    is_class(x, cls, exception)

def is_struct(x, cls, exception):
    if not isinstance(x, Structure):
        raise exception("'%s' is not a structure" % (x))
    is_class(x, cls, exception)

def is_enum(x, cls, exception):
    if not isinstance(x, Enumeration):
        raise exception("'%s' is not an enumeration" % (x))
    is_class(x, cls, exception)

def is_valobj(x, cls, exception):
    if x == None:
    	return True # ValueObject may be null / 'None'
    if not isinstance(x, ValueObject):
        raise exception("'%s' is not a value object" % (x))
    is_class(x, cls, exception)

#
# helpers
#
def is_class(x, cls, exception):
    if not isinstance(x, cls):
        raise exception("'%s' is not a %s" % (x, cls))



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "SnmpV3.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# interface
class SnmpV3(Interface):
    idlType = "um.SnmpV3:1.0.0"

    # enumeration
    class SecurityLevel(Enumeration):
        idlType = "um.SnmpV3.SecurityLevel:1.0.0"
        values = ["NO_AUTH_NO_PRIV", "AUTH_NO_PRIV", "AUTH_PRIV"]

    SecurityLevel.NO_AUTH_NO_PRIV = SecurityLevel(0)
    SecurityLevel.AUTH_NO_PRIV = SecurityLevel(1)
    SecurityLevel.AUTH_PRIV = SecurityLevel(2)

    # enumeration
    class AuthProtocol(Enumeration):
        idlType = "um.SnmpV3.AuthProtocol:1.0.0"
        values = ["MD5", "SHA1"]

    AuthProtocol.MD5 = AuthProtocol(0)
    AuthProtocol.SHA1 = AuthProtocol(1)

    # enumeration
    class PrivProtocol(Enumeration):
        idlType = "um.SnmpV3.PrivProtocol:1.0.0"
        values = ["DES", "AES128"]

    PrivProtocol.DES = PrivProtocol(0)
    PrivProtocol.AES128 = PrivProtocol(1)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Event.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# value object
class Event(ValueObject):
    idlType = "idl.Event:1.0.0"

    def __init__(self, source):
        typecheck.is_remote_obj(source, AssertionError)

        self.source = source

    def encode(self):
        json = {}
        json['source'] = Interface.encode(self.source)
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = ["source"]
        return elements



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Log.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# structure
class LogInfo(Structure):
    idlType = "logging.LogInfo:1.0.0"
    elements = ["creationTime", "idFirst", "idNext"]

    def __init__(self, creationTime, idFirst, idNext):
        typecheck.is_long(creationTime, AssertionError)
        typecheck.is_int(idFirst, AssertionError)
        typecheck.is_int(idNext, AssertionError)

        self.creationTime = creationTime
        self.idFirst = idFirst
        self.idNext = idNext

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            creationTime = int(json['creationTime']),
            idFirst = json['idFirst'],
            idNext = json['idNext'],
        )
        return obj

    def encode(self):
        json = {}
        json['creationTime'] = self.creationTime
        json['idFirst'] = self.idFirst
        json['idNext'] = self.idNext
        return json

# structure
class LogEntry(Structure):
    idlType = "logging.LogEntry:1.0.0"
    elements = ["id", "timestamp", "eventClass", "message"]

    def __init__(self, id, timestamp, eventClass, message):
        typecheck.is_int(id, AssertionError)
        typecheck.is_time(timestamp, AssertionError)
        typecheck.is_string(eventClass, AssertionError)
        typecheck.is_string(message, AssertionError)

        self.id = id
        self.timestamp = timestamp
        self.eventClass = eventClass
        self.message = message

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            id = json['id'],
            timestamp = raritan.rpc.Time.decode(json['timestamp']),
            eventClass = json['eventClass'],
            message = json['message'],
        )
        return obj

    def encode(self):
        json = {}
        json['id'] = self.id
        json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
        json['eventClass'] = self.eventClass
        json['message'] = self.message
        return json

# structure
class LogChunk(Structure):
    idlType = "logging.LogChunk:1.0.0"
    elements = ["logCreationTime", "idFirst", "allEntryCnt", "selEntries"]

    def __init__(self, logCreationTime, idFirst, allEntryCnt, selEntries):
        typecheck.is_long(logCreationTime, AssertionError)
        typecheck.is_int(idFirst, AssertionError)
        typecheck.is_int(allEntryCnt, AssertionError)
        for x0 in selEntries:
            typecheck.is_struct(x0, raritan.rpc.logging.LogEntry, AssertionError)

        self.logCreationTime = logCreationTime
        self.idFirst = idFirst
        self.allEntryCnt = allEntryCnt
        self.selEntries = selEntries

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            logCreationTime = int(json['logCreationTime']),
            idFirst = json['idFirst'],
            allEntryCnt = json['allEntryCnt'],
            selEntries = [raritan.rpc.logging.LogEntry.decode(x0, agent) for x0 in json['selEntries']],
        )
        return obj

    def encode(self):
        json = {}
        json['logCreationTime'] = self.logCreationTime
        json['idFirst'] = self.idFirst
        json['allEntryCnt'] = self.allEntryCnt
        json['selEntries'] = [raritan.rpc.logging.LogEntry.encode(x0) for x0 in self.selEntries]
        return json

# enumeration
class RangeDirection(Enumeration):
    idlType = "logging.RangeDirection:1.0.0"
    values = ["FORWARD", "BACKWARD"]

RangeDirection.FORWARD = RangeDirection(0)
RangeDirection.BACKWARD = RangeDirection(1)

#
# Section generated by IdlC from "EventLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.logging


# value object
class EventLogClearedEvent(raritan.rpc.event.UserEvent):
    idlType = "logging.EventLogClearedEvent:1.0.0"

    def __init__(self, actUserName, actIpAddr, source):
        super(raritan.rpc.logging.EventLogClearedEvent, self).__init__(actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.logging.EventLogClearedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            # for event.UserEvent
            actUserName = json['actUserName'],
            actIpAddr = json['actIpAddr'],
            # for idl.Event
            source = Interface.decode(json['source'], agent),
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.logging.EventLogClearedEvent, self).listElements()
        return elements

# interface
class EventLog(Interface):
    idlType = "logging.EventLog:2.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction, categories):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            for x0 in categories:
                typecheck.is_string(x0, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            args['categories'] = [x0 for x0 in categories]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(EventLog, self).__init__(target, agent)
        self.clear = EventLog._clear(self)
        self.getInfo = EventLog._getInfo(self)
        self.getChunk = EventLog._getChunk(self)

#
# Section generated by IdlC from "WlanLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# interface
class WlanLog(Interface):
    idlType = "logging.WlanLog:1.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(WlanLog, self).__init__(target, agent)
        self.clear = WlanLog._clear(self)
        self.getInfo = WlanLog._getInfo(self)
        self.getChunk = WlanLog._getChunk(self)

#
# Section generated by IdlC from "DebugLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# interface
class DebugLog(Interface):
    idlType = "logging.DebugLog:2.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DebugLog, self).__init__(target, agent)
        self.clear = DebugLog._clear(self)
        self.getInfo = DebugLog._getInfo(self)
        self.getChunk = DebugLog._getChunk(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Fitness.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.fitness


# interface
class Fitness(Interface):
    idlType = "fitness.Fitness:1.0.0"

    FLAG_VALUE_INVALID = 0x1

    FLAG_VALUE_OLD = 0x2

    FLAG_ENTRY_CRITICAL = 0x4

    # structure
    class DataEntry(Structure):
        idlType = "fitness.Fitness.DataEntry:1.0.0"
        elements = ["id", "value", "maxValue", "worstValue", "thresholdValue", "rawValue", "flags"]

        def __init__(self, id, value, maxValue, worstValue, thresholdValue, rawValue, flags):
            typecheck.is_string(id, AssertionError)
            typecheck.is_int(value, AssertionError)
            typecheck.is_int(maxValue, AssertionError)
            typecheck.is_int(worstValue, AssertionError)
            typecheck.is_int(thresholdValue, AssertionError)
            typecheck.is_long(rawValue, AssertionError)
            typecheck.is_int(flags, AssertionError)

            self.id = id
            self.value = value
            self.maxValue = maxValue
            self.worstValue = worstValue
            self.thresholdValue = thresholdValue
            self.rawValue = rawValue
            self.flags = flags

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                value = json['value'],
                maxValue = json['maxValue'],
                worstValue = json['worstValue'],
                thresholdValue = json['thresholdValue'],
                rawValue = int(json['rawValue']),
                flags = json['flags'],
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = self.id
            json['value'] = self.value
            json['maxValue'] = self.maxValue
            json['worstValue'] = self.worstValue
            json['thresholdValue'] = self.thresholdValue
            json['rawValue'] = self.rawValue
            json['flags'] = self.flags
            return json

    # structure
    class ErrorLogEntry(Structure):
        idlType = "fitness.Fitness.ErrorLogEntry:1.0.0"
        elements = ["id", "value", "thresholdValue", "rawValue", "powerOnHours", "timeStampUTC"]

        def __init__(self, id, value, thresholdValue, rawValue, powerOnHours, timeStampUTC):
            typecheck.is_string(id, AssertionError)
            typecheck.is_int(value, AssertionError)
            typecheck.is_int(thresholdValue, AssertionError)
            typecheck.is_long(rawValue, AssertionError)
            typecheck.is_int(powerOnHours, AssertionError)
            typecheck.is_time(timeStampUTC, AssertionError)

            self.id = id
            self.value = value
            self.thresholdValue = thresholdValue
            self.rawValue = rawValue
            self.powerOnHours = powerOnHours
            self.timeStampUTC = timeStampUTC

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                value = json['value'],
                thresholdValue = json['thresholdValue'],
                rawValue = int(json['rawValue']),
                powerOnHours = json['powerOnHours'],
                timeStampUTC = raritan.rpc.Time.decode(json['timeStampUTC']),
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = self.id
            json['value'] = self.value
            json['thresholdValue'] = self.thresholdValue
            json['rawValue'] = self.rawValue
            json['powerOnHours'] = self.powerOnHours
            json['timeStampUTC'] = raritan.rpc.Time.encode(self.timeStampUTC)
            return json

    class _getDataEntries(Interface.Method):
        name = 'getDataEntries'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.fitness.Fitness.DataEntry.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.fitness.Fitness.DataEntry, DecodeException)
            return _ret_

    class _getErrorLogIndexRange(Interface.Method):
        name = 'getErrorLogIndexRange'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            firstIndex = rsp['firstIndex']
            entryCount = rsp['entryCount']
            typecheck.is_int(firstIndex, DecodeException)
            typecheck.is_int(entryCount, DecodeException)
            return (firstIndex, entryCount)

    class _getErrorLogEntries(Interface.Method):
        name = 'getErrorLogEntries'

        @staticmethod
        def encode(startIndex, count):
            typecheck.is_int(startIndex, AssertionError)
            typecheck.is_int(count, AssertionError)
            args = {}
            args['startIndex'] = startIndex
            args['count'] = count
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.fitness.Fitness.ErrorLogEntry.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.fitness.Fitness.ErrorLogEntry, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Fitness, self).__init__(target, agent)
        self.getDataEntries = Fitness._getDataEntries(self)
        self.getErrorLogIndexRange = Fitness._getErrorLogIndexRange(self)
        self.getErrorLogEntries = Fitness._getErrorLogEntries(self)

#
# Section generated by IdlC from "HardwareHealth.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.fitness

import raritan.rpc.idl


# interface
class HardwareHealth(Interface):
    idlType = "fitness.HardwareHealth:1.0.1"

    FAILURE_TYPE_NETWORK_DEVICE_NOT_DETECTED = 1

    FAILURE_TYPE_I2C_BUS_STUCK = 2

    FAILURE_TYPE_SLAVE_CTRL_NOT_REACHABLE = 3

    FAILURE_TYPE_SLAVE_CTRL_MALFUNCTION = 4

    FAILURE_TYPE_OUTLET_POWER_STATE_INCONSISTENT = 5

    # structure
    class Failure(Structure):
        idlType = "fitness.HardwareHealth_1_0_1.Failure:1.0.0"
        elements = ["creationOrderId", "componentId", "type", "description", "isAsserted", "lastAssertTimeStamp", "lastDeassertTimeStamp", "assertCount"]

        def __init__(self, creationOrderId, componentId, type, description, isAsserted, lastAssertTimeStamp, lastDeassertTimeStamp, assertCount):
            typecheck.is_int(creationOrderId, AssertionError)
            typecheck.is_string(componentId, AssertionError)
            typecheck.is_int(type, AssertionError)
            typecheck.is_string(description, AssertionError)
            typecheck.is_bool(isAsserted, AssertionError)
            typecheck.is_time(lastAssertTimeStamp, AssertionError)
            typecheck.is_time(lastDeassertTimeStamp, AssertionError)
            typecheck.is_int(assertCount, AssertionError)

            self.creationOrderId = creationOrderId
            self.componentId = componentId
            self.type = type
            self.description = description
            self.isAsserted = isAsserted
            self.lastAssertTimeStamp = lastAssertTimeStamp
            self.lastDeassertTimeStamp = lastDeassertTimeStamp
            self.assertCount = assertCount

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                creationOrderId = json['creationOrderId'],
                componentId = json['componentId'],
                type = json['type'],
                description = json['description'],
                isAsserted = json['isAsserted'],
                lastAssertTimeStamp = raritan.rpc.Time.decode(json['lastAssertTimeStamp']),
                lastDeassertTimeStamp = raritan.rpc.Time.decode(json['lastDeassertTimeStamp']),
                assertCount = json['assertCount'],
            )
            return obj

        def encode(self):
            json = {}
            json['creationOrderId'] = self.creationOrderId
            json['componentId'] = self.componentId
            json['type'] = self.type
            json['description'] = self.description
            json['isAsserted'] = self.isAsserted
            json['lastAssertTimeStamp'] = raritan.rpc.Time.encode(self.lastAssertTimeStamp)
            json['lastDeassertTimeStamp'] = raritan.rpc.Time.encode(self.lastDeassertTimeStamp)
            json['assertCount'] = self.assertCount
            return json

    # value object
    class FailureStatusChangedEvent(raritan.rpc.idl.Event):
        idlType = "fitness.HardwareHealth_1_0_1.FailureStatusChangedEvent:1.0.0"

        def __init__(self, componentId, failureType, isAsserted, source):
            super(raritan.rpc.fitness.HardwareHealth.FailureStatusChangedEvent, self).__init__(source)
            typecheck.is_string(componentId, AssertionError)
            typecheck.is_int(failureType, AssertionError)
            typecheck.is_bool(isAsserted, AssertionError)

            self.componentId = componentId
            self.failureType = failureType
            self.isAsserted = isAsserted

        def encode(self):
            json = super(raritan.rpc.fitness.HardwareHealth.FailureStatusChangedEvent, self).encode()
            json['componentId'] = self.componentId
            json['failureType'] = self.failureType
            json['isAsserted'] = self.isAsserted
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                componentId = json['componentId'],
                failureType = json['failureType'],
                isAsserted = json['isAsserted'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["componentId", "failureType", "isAsserted"]
            elements = elements + super(raritan.rpc.fitness.HardwareHealth.FailureStatusChangedEvent, self).listElements()
            return elements

    class _getFailures(Interface.Method):
        name = 'getFailures'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.fitness.HardwareHealth.Failure.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.fitness.HardwareHealth.Failure, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(HardwareHealth, self).__init__(target, agent)
        self.getFailures = HardwareHealth._getFailures(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "ScannerCtrl.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# interface
class ScannerCtrl(Interface):
    idlType = "tfw.ScannerCtrl:1.0.0"

    class _getScanInterval(Interface.Method):
        name = 'getScanInterval'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setScanInterval(Interface.Method):
        name = 'setScanInterval'

        @staticmethod
        def encode(interval):
            typecheck.is_int(interval, AssertionError)
            args = {}
            args['interval'] = interval
            return args

        @staticmethod
        def decode(rsp, agent):
            return None
    def __init__(self, target, agent):
        super(ScannerCtrl, self).__init__(target, agent)
        self.getScanInterval = ScannerCtrl._getScanInterval(self)
        self.setScanInterval = ScannerCtrl._setScanInterval(self)

#
# Section generated by IdlC from "CoreCtrl.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.tfw


# interface
class CoreCtrl(Interface):
    idlType = "tfw.CoreCtrl:1.0.0"

    class _getScanner(Interface.Method):
        name = 'getScanner'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_interface(_ret_, raritan.rpc.tfw.ScannerCtrl, DecodeException)
            return _ret_

    class _startAllScanners(Interface.Method):
        name = 'startAllScanners'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _stopAllScanners(Interface.Method):
        name = 'stopAllScanners'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _startAllBackWorkers(Interface.Method):
        name = 'startAllBackWorkers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _stopAllBackWorkers(Interface.Method):
        name = 'stopAllBackWorkers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None
    def __init__(self, target, agent):
        super(CoreCtrl, self).__init__(target, agent)
        self.getScanner = CoreCtrl._getScanner(self)
        self.startAllScanners = CoreCtrl._startAllScanners(self)
        self.stopAllScanners = CoreCtrl._stopAllScanners(self)
        self.startAllBackWorkers = CoreCtrl._startAllBackWorkers(self)
        self.stopAllBackWorkers = CoreCtrl._stopAllBackWorkers(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "LdapServerSettings.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

#
# Section generated by IdlC from "RadiusManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.radius


# interface
class RadiusManager(Interface):
    idlType = "auth.RadiusManager:2.0.0"

    ERR_INVALID_CFG = 1

    ERR_SERVER_UNSPECIFIED = 2

    ERR_INVALID_SHARED_SECRET = 3

    ERR_SERVER_UNREACHABLE = 4

    ERR_AUTHENTICATION_FAILED = 5

    ERR_NO_ROLES = 6

    ERR_NO_KNOWN_ROLES = 7

    class _getRadiusServers(Interface.Method):
        name = 'getRadiusServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.radius.ServerSettings.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.radius.ServerSettings, DecodeException)
            return _ret_

    class _setRadiusServers(Interface.Method):
        name = 'setRadiusServers'

        @staticmethod
        def encode(serverList):
            for x0 in serverList:
                typecheck.is_struct(x0, raritan.rpc.radius.ServerSettings, AssertionError)
            args = {}
            args['serverList'] = [raritan.rpc.radius.ServerSettings.encode(x0) for x0 in serverList]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _testRadiusServer(Interface.Method):
        name = 'testRadiusServer'

        @staticmethod
        def encode(username, password, settings):
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.radius.ServerSettings, AssertionError)
            args = {}
            args['username'] = username
            args['password'] = password
            args['settings'] = raritan.rpc.radius.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(RadiusManager, self).__init__(target, agent)
        self.getRadiusServers = RadiusManager._getRadiusServers(self)
        self.setRadiusServers = RadiusManager._setRadiusServers(self)
        self.testRadiusServer = RadiusManager._testRadiusServer(self)

#
# Section generated by IdlC from "AuthManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth


# enumeration
class Type(Enumeration):
    idlType = "auth.Type:1.0.0"
    values = ["LOCAL", "RADIUS", "KERBEROS", "TACACS_PLUS", "LDAP"]

Type.LOCAL = Type(0)
Type.RADIUS = Type(1)
Type.KERBEROS = Type(2)
Type.TACACS_PLUS = Type(3)
Type.LDAP = Type(4)

# structure
class Policy(Structure):
    idlType = "auth.Policy:1.0.0"
    elements = ["type", "useLocalIfRemoteFailed"]

    def __init__(self, type, useLocalIfRemoteFailed):
        typecheck.is_enum(type, raritan.rpc.auth.Type, AssertionError)
        typecheck.is_bool(useLocalIfRemoteFailed, AssertionError)

        self.type = type
        self.useLocalIfRemoteFailed = useLocalIfRemoteFailed

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            type = raritan.rpc.auth.Type.decode(json['type']),
            useLocalIfRemoteFailed = json['useLocalIfRemoteFailed'],
        )
        return obj

    def encode(self):
        json = {}
        json['type'] = raritan.rpc.auth.Type.encode(self.type)
        json['useLocalIfRemoteFailed'] = self.useLocalIfRemoteFailed
        return json

# interface
class AuthManager(Interface):
    idlType = "auth.AuthManager:1.0.0"

    ERR_UNSUPPORTED_TYPE = 1

    class _getPolicy(Interface.Method):
        name = 'getPolicy'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.auth.Policy.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.auth.Policy, DecodeException)
            return _ret_

    class _setPolicy(Interface.Method):
        name = 'setPolicy'

        @staticmethod
        def encode(p):
            typecheck.is_struct(p, raritan.rpc.auth.Policy, AssertionError)
            args = {}
            args['p'] = raritan.rpc.auth.Policy.encode(p)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(AuthManager, self).__init__(target, agent)
        self.getPolicy = AuthManager._getPolicy(self)
        self.setPolicy = AuthManager._setPolicy(self)

#
# Section generated by IdlC from "LdapManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth.ldapsrv


# interface
class LdapManager(Interface):
    idlType = "auth.LdapManager:2.0.0"

    class _getLdapServers(Interface.Method):
        name = 'getLdapServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.auth.ldapsrv.ServerSettings.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.auth.ldapsrv.ServerSettings, DecodeException)
            return _ret_

    ERR_CYCLIC_DEP = 1

    ERR_INVALID_CFG = 2

    class _setLdapServers(Interface.Method):
        name = 'setLdapServers'

        @staticmethod
        def encode(serverList):
            for x0 in serverList:
                typecheck.is_struct(x0, raritan.rpc.auth.ldapsrv.ServerSettings, AssertionError)
            args = {}
            args['serverList'] = [raritan.rpc.auth.ldapsrv.ServerSettings.encode(x0) for x0 in serverList]
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    ERR_SERVER_UNSPECIFIED = 1

    ERR_SERVER_UNREACHABLE = 3

    ERR_AUTHENTICATION_FAILED = 4

    ERR_NO_ROLES = 5

    ERR_NO_KNOWN_ROLES = 6

    class _testLdapServer(Interface.Method):
        name = 'testLdapServer'

        @staticmethod
        def encode(username, password, settings):
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.auth.ldapsrv.ServerSettings, AssertionError)
            args = {}
            args['username'] = username
            args['password'] = password
            args['settings'] = raritan.rpc.auth.ldapsrv.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            diagMsg = rsp['diagMsg']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_string(diagMsg, DecodeException)
            return (_ret_, diagMsg)
    def __init__(self, target, agent):
        super(LdapManager, self).__init__(target, agent)
        self.getLdapServers = LdapManager._getLdapServers(self)
        self.setLdapServers = LdapManager._setLdapServers(self)
        self.testLdapServer = LdapManager._testLdapServer(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "DiagLogSettings.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.diag


# interface
class DiagLogSettings(Interface):
    idlType = "diag.DiagLogSettings:1.0.0"

    # enumeration
    class LogLevel(Enumeration):
        idlType = "diag.DiagLogSettings.LogLevel:1.0.0"
        values = ["LOG_LEVEL_NONE", "LOG_LEVEL_ERR", "LOG_LEVEL_WARN", "LOG_LEVEL_INFO", "LOG_LEVEL_DEBUG", "LOG_LEVEL_TRACE"]

    LogLevel.LOG_LEVEL_NONE = LogLevel(0)
    LogLevel.LOG_LEVEL_ERR = LogLevel(1)
    LogLevel.LOG_LEVEL_WARN = LogLevel(2)
    LogLevel.LOG_LEVEL_INFO = LogLevel(3)
    LogLevel.LOG_LEVEL_DEBUG = LogLevel(4)
    LogLevel.LOG_LEVEL_TRACE = LogLevel(5)

    # structure
    class LogLevelEntry(Structure):
        idlType = "diag.DiagLogSettings.LogLevelEntry:1.0.0"
        elements = ["ctxName", "logLevel"]

        def __init__(self, ctxName, logLevel):
            typecheck.is_string(ctxName, AssertionError)
            typecheck.is_enum(logLevel, raritan.rpc.diag.DiagLogSettings.LogLevel, AssertionError)

            self.ctxName = ctxName
            self.logLevel = logLevel

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                ctxName = json['ctxName'],
                logLevel = raritan.rpc.diag.DiagLogSettings.LogLevel.decode(json['logLevel']),
            )
            return obj

        def encode(self):
            json = {}
            json['ctxName'] = self.ctxName
            json['logLevel'] = raritan.rpc.diag.DiagLogSettings.LogLevel.encode(self.logLevel)
            return json

    ERR_NONE = 0

    ERR_UNKNOWN_LOG_CONTEXT_NAME = 1

    ERR_UNKNOWN_LOG_LEVEL = 2

    class _resetLogLevelsForAllCtxNames(Interface.Method):
        name = 'resetLogLevelsForAllCtxNames'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _getLogLevelsForAllCtxNames(Interface.Method):
        name = 'getLogLevelsForAllCtxNames'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.diag.DiagLogSettings.LogLevelEntry.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.diag.DiagLogSettings.LogLevelEntry, DecodeException)
            return _ret_

    class _setLogLevelForAllCtxNames(Interface.Method):
        name = 'setLogLevelForAllCtxNames'

        @staticmethod
        def encode(logLevel):
            typecheck.is_enum(logLevel, raritan.rpc.diag.DiagLogSettings.LogLevel, AssertionError)
            args = {}
            args['logLevel'] = raritan.rpc.diag.DiagLogSettings.LogLevel.encode(logLevel)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getLogLevelByCtxName(Interface.Method):
        name = 'getLogLevelByCtxName'

        @staticmethod
        def encode(ctxName):
            typecheck.is_string(ctxName, AssertionError)
            args = {}
            args['ctxName'] = ctxName
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            logLevel = raritan.rpc.diag.DiagLogSettings.LogLevel.decode(rsp['logLevel'])
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_enum(logLevel, raritan.rpc.diag.DiagLogSettings.LogLevel, DecodeException)
            return (_ret_, logLevel)

    class _setLogLevelByCtxName(Interface.Method):
        name = 'setLogLevelByCtxName'

        @staticmethod
        def encode(ctxName, logLevel):
            typecheck.is_string(ctxName, AssertionError)
            typecheck.is_enum(logLevel, raritan.rpc.diag.DiagLogSettings.LogLevel, AssertionError)
            args = {}
            args['ctxName'] = ctxName
            args['logLevel'] = raritan.rpc.diag.DiagLogSettings.LogLevel.encode(logLevel)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DiagLogSettings, self).__init__(target, agent)
        self.resetLogLevelsForAllCtxNames = DiagLogSettings._resetLogLevelsForAllCtxNames(self)
        self.getLogLevelsForAllCtxNames = DiagLogSettings._getLogLevelsForAllCtxNames(self)
        self.setLogLevelForAllCtxNames = DiagLogSettings._setLogLevelForAllCtxNames(self)
        self.getLogLevelByCtxName = DiagLogSettings._getLogLevelByCtxName(self)
        self.setLogLevelByCtxName = DiagLogSettings._setLogLevelByCtxName(self)

# from raritan/rpc/diag/__extend__.py
def download_diag(agent):
    """
    Method to download the diagnostic data

    **parameters**

    :param agent: An agent instance from the device where the diagnostic file should be downloaded
    :return: returns the diagnostic data

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import diag

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        # download
        diag = diag.download_diag(agent)
        print(diag)
    """
    target = "cgi-bin/diag-data.cgi"
    return agent.get(target)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "BulkConfiguration.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.bulkcfg

import raritan.rpc.event


# interface
class BulkConfiguration(Interface):
    idlType = "bulkcfg.BulkConfiguration:1.0.2"

    # enumeration
    class Status(Enumeration):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.Status:1.0.0"
        values = ["UNKNOWN", "UPLOAD_FAILED", "RESTORE_PENDING", "RESTORE_OK", "RESTORE_FAILED"]

    Status.UNKNOWN = Status(0)
    Status.UPLOAD_FAILED = Status(1)
    Status.RESTORE_PENDING = Status(2)
    Status.RESTORE_OK = Status(3)
    Status.RESTORE_FAILED = Status(4)

    class _getStatus(Interface.Method):
        name = 'getStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            status = raritan.rpc.bulkcfg.BulkConfiguration.Status.decode(rsp['status'])
            timeStamp = raritan.rpc.Time.decode(rsp['timeStamp'])
            typecheck.is_enum(status, raritan.rpc.bulkcfg.BulkConfiguration.Status, DecodeException)
            typecheck.is_time(timeStamp, DecodeException)
            return (status, timeStamp)

    # enumeration
    class FilterType(Enumeration):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.FilterType:1.0.0"
        values = ["WHITELIST", "BLACKLIST"]

    FilterType.WHITELIST = FilterType(0)
    FilterType.BLACKLIST = FilterType(1)

    # structure
    class Filter(Structure):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.Filter:1.0.0"
        elements = ["name", "displayName", "noOverride", "bulkOnly", "ruleSpecs"]

        def __init__(self, name, displayName, noOverride, bulkOnly, ruleSpecs):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(displayName, AssertionError)
            typecheck.is_bool(noOverride, AssertionError)
            typecheck.is_bool(bulkOnly, AssertionError)
            for x0 in ruleSpecs:
                typecheck.is_string(x0, AssertionError)

            self.name = name
            self.displayName = displayName
            self.noOverride = noOverride
            self.bulkOnly = bulkOnly
            self.ruleSpecs = ruleSpecs

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                displayName = json['displayName'],
                noOverride = json['noOverride'],
                bulkOnly = json['bulkOnly'],
                ruleSpecs = [x0 for x0 in json['ruleSpecs']],
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['displayName'] = self.displayName
            json['noOverride'] = self.noOverride
            json['bulkOnly'] = self.bulkOnly
            json['ruleSpecs'] = [x0 for x0 in self.ruleSpecs]
            return json

    # structure
    class FilterProfile(Structure):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.FilterProfile:1.0.0"
        elements = ["name", "description", "filterNameToTypeMap"]

        def __init__(self, name, description, filterNameToTypeMap):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(description, AssertionError)

            self.name = name
            self.description = description
            self.filterNameToTypeMap = filterNameToTypeMap

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                description = json['description'],
                filterNameToTypeMap = dict([(
                    elem['key'],
                    raritan.rpc.bulkcfg.BulkConfiguration.FilterType.decode(elem['value']))
                    for elem in json['filterNameToTypeMap']]),
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['description'] = self.description
            json['filterNameToTypeMap'] = [dict(
                key = k,
                value = raritan.rpc.bulkcfg.BulkConfiguration.FilterType.encode(v))
                for k, v in self.filterNameToTypeMap.items()]
            return json

    # structure
    class Settings(Structure):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.Settings:1.0.0"
        elements = ["filterProfiles", "defaultProfileName"]

        def __init__(self, filterProfiles, defaultProfileName):
            for x0 in filterProfiles:
                typecheck.is_struct(x0, raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile, AssertionError)
            typecheck.is_string(defaultProfileName, AssertionError)

            self.filterProfiles = filterProfiles
            self.defaultProfileName = defaultProfileName

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                filterProfiles = [raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile.decode(x0, agent) for x0 in json['filterProfiles']],
                defaultProfileName = json['defaultProfileName'],
            )
            return obj

        def encode(self):
            json = {}
            json['filterProfiles'] = [raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile.encode(x0) for x0 in self.filterProfiles]
            json['defaultProfileName'] = self.defaultProfileName
            return json

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.SettingsChangedEvent:1.0.0"

        def __init__(self, actUserName, actIpAddr, source):
            super(raritan.rpc.bulkcfg.BulkConfiguration.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)

        def encode(self):
            json = super(raritan.rpc.bulkcfg.BulkConfiguration.SettingsChangedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.bulkcfg.BulkConfiguration.SettingsChangedEvent, self).listElements()
            return elements

    # value object
    class SavedEvent(raritan.rpc.event.UserEvent):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.SavedEvent:1.0.0"

        def __init__(self, isBackup, actUserName, actIpAddr, source):
            super(raritan.rpc.bulkcfg.BulkConfiguration.SavedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_bool(isBackup, AssertionError)

            self.isBackup = isBackup

        def encode(self):
            json = super(raritan.rpc.bulkcfg.BulkConfiguration.SavedEvent, self).encode()
            json['isBackup'] = self.isBackup
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                isBackup = json['isBackup'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["isBackup"]
            elements = elements + super(raritan.rpc.bulkcfg.BulkConfiguration.SavedEvent, self).listElements()
            return elements

    # value object
    class RestoredEvent(raritan.rpc.event.UserEvent):
        idlType = "bulkcfg.BulkConfiguration_1_0_2.RestoredEvent:1.0.0"

        def __init__(self, isBackup, actUserName, actIpAddr, source):
            super(raritan.rpc.bulkcfg.BulkConfiguration.RestoredEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_bool(isBackup, AssertionError)

            self.isBackup = isBackup

        def encode(self):
            json = super(raritan.rpc.bulkcfg.BulkConfiguration.RestoredEvent, self).encode()
            json['isBackup'] = self.isBackup
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                isBackup = json['isBackup'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["isBackup"]
            elements = elements + super(raritan.rpc.bulkcfg.BulkConfiguration.RestoredEvent, self).listElements()
            return elements

    SUCCESS = 0

    ERR_FILTER_NAME_UNKNOWN = 1

    ERR_FILTER_TYPE_READONLY = 2

    ERR_PROFILE_ALREADY_EXISTS = 3

    ERR_PROFILE_DOES_NOT_EXIST = 4

    ERR_PROFILE_IS_DEFAULT = 5

    ERR_PROFILE_IS_BUILTIN = 6

    ERR_PROFILE_NAME_TOO_LONG = 7

    ERR_PROFILE_NAME_INVALID = 8

    ERR_PROFILE_TOO_MANY = 9

    class _getFilters(Interface.Method):
        name = 'getFilters'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.bulkcfg.BulkConfiguration.Filter.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.bulkcfg.BulkConfiguration.Filter, DecodeException)
            return _ret_

    class _getFilterProfiles(Interface.Method):
        name = 'getFilterProfiles'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile, DecodeException)
            return _ret_

    class _addFilterProfile(Interface.Method):
        name = 'addFilterProfile'

        @staticmethod
        def encode(profile):
            typecheck.is_struct(profile, raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile, AssertionError)
            args = {}
            args['profile'] = raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile.encode(profile)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _modifyFilterProfile(Interface.Method):
        name = 'modifyFilterProfile'

        @staticmethod
        def encode(profile):
            typecheck.is_struct(profile, raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile, AssertionError)
            args = {}
            args['profile'] = raritan.rpc.bulkcfg.BulkConfiguration.FilterProfile.encode(profile)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _deleteFilterProfile(Interface.Method):
        name = 'deleteFilterProfile'

        @staticmethod
        def encode(profileName):
            typecheck.is_string(profileName, AssertionError)
            args = {}
            args['profileName'] = profileName
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getDefaultFilterProfileName(Interface.Method):
        name = 'getDefaultFilterProfileName'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _selectDefaultFilterProfile(Interface.Method):
        name = 'selectDefaultFilterProfile'

        @staticmethod
        def encode(profileName):
            typecheck.is_string(profileName, AssertionError)
            args = {}
            args['profileName'] = profileName
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.bulkcfg.BulkConfiguration.Settings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.bulkcfg.BulkConfiguration.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.bulkcfg.BulkConfiguration.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.bulkcfg.BulkConfiguration.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(BulkConfiguration, self).__init__(target, agent)
        self.getStatus = BulkConfiguration._getStatus(self)
        self.getFilters = BulkConfiguration._getFilters(self)
        self.getFilterProfiles = BulkConfiguration._getFilterProfiles(self)
        self.addFilterProfile = BulkConfiguration._addFilterProfile(self)
        self.modifyFilterProfile = BulkConfiguration._modifyFilterProfile(self)
        self.deleteFilterProfile = BulkConfiguration._deleteFilterProfile(self)
        self.getDefaultFilterProfileName = BulkConfiguration._getDefaultFilterProfileName(self)
        self.selectDefaultFilterProfile = BulkConfiguration._selectDefaultFilterProfile(self)
        self.getSettings = BulkConfiguration._getSettings(self)
        self.setSettings = BulkConfiguration._setSettings(self)

# from raritan/rpc/bulkcfg/__extend__.py
def upload(agent, data, full = False, backup = False):
    """
    Method to upload bulk config files.
    Check the according response header X-Response-Code for succesfull configuration.

    **parameters**, **return**

    :param agent: An agent instance for the device where the config should be uploaded
    :param data: The binary data of the bulk config file
    :param backup: (optional) Is the file a settings backup?
    :return: return the response code from internal processing

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import bulkcfg

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")

        # read file in binary mode
        cfgFile = open("bulk_config.txt", "rb")
        # upload
        code = bulkcfg.upload(agent, cfgFile.read())
        # view code
        print(code)
    """
    target = "cgi-bin/bulk_config_restore.cgi"
    if full:
        target += "?mode=full"
    elif backup:
        target += "?mode=backup"
    response = agent.form_data_file(target, [data], ["bulk_config.txt"], ["bulk_config_file"], ["application/octet-stream"])
    return response.headers.get("X-Response-Code")

def download_bulkcfg(agent, full = False, backup = False):
    """
    Method to download the bulk config file

    **parameters**

    :param agent: An agent instance from the device where the bulk config should be downloaded
    :param full: (optional) The full configuration?
    :param backup: (optional) Only the configuration backup?
    :return: returns the bulk configuration data

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import bulkcfg

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        # download
        cfg = bulkcfg.download_bulkcfg(agent)
        print(cfg)
    """
    target = "cgi-bin/bulk_config_save.cgi"
    if full:
        target += "?mode=full"
    elif backup:
        target += "?mode=backup"
    return agent.get(target)
